#!/usr/bin/perl

use strict;
use warnings;
use vars qw(@MODULES);

use Test::More;

eval "use Test::Pod 1.00";

plan skip_all => "Test::Pod 1.00 required for testing POD" if $@;

@MODULES = all_pod_files();
plan tests => scalar(@MODULES);

pod_file_ok($_) for (@MODULES);

exit;
